#include <mp4lib/mp4.h>

int atom_head(FILE *fh, atom_t *atom)
{
  unsigned char headBuf[8];

  /* Get the header (4 bytes size, 4 bytes type string) */
  if (fread(headBuf, 1, 8, fh) < 8) {
    return 0; /* Something went wrong with the read */
  }
  /* Put the header's info into the atom */
  atom->size = str2int(headBuf, 4);
  memcpy(atom->name, headBuf + 4, 4);
  atom->pos = 8;

  /* Is this a placeholder atom for potential wide-ness? */
  if (atomid(atom->name) == atomid("wide")) {
    /* Yes. Now we have to get the real header. */
    /* Get the header (4 bytes size, 4 bytes type string) */
    if (fread(headBuf, 1, 8, fh) < 8) {
      return 0; /* Something went wrong with the read */
    }
    /* Put the header's info into the atom */
    atom->size = str2int(headBuf, 4);
    memcpy(atom->name, headBuf + 4, 4);
    atom->pos += 8;
  } else if (atom->size == 1) { /* Extended 64-bit length */
    if (fread(headBuf, 1, 8, fh) < 8) {
      return 0;
    }
    atom->size = str2int(headBuf, 8);
  }
  atom->name[4] = '\0';
#ifdef DEBUG
  printf("atom_head: Atom type %s,", atom->name);
  printf(" size %u,", atom->size);
  printf(" current position %u.\n", atom->pos);
#endif
  return 1;
}

void skip_atom(FILE *fh, atom_t *atom)
{
#ifdef DEBUG
  printf("skip_atom: Atom type %c%c%c%c,",
          atom->name[0], atom->name[1], atom->name[2], atom->name[3]);
  printf(" size %u,", atom->size);
  printf(" current position %u.\n", atom->pos);
#endif
 if (atom->size - atom->pos) { /* Don't bother to advance if already at end */
    fseek(fh, atom->size - atom->pos, SEEK_CUR);
    atom->pos = atom->size;
#ifdef DEBUG
  printf("skip_atom: Atom type %c%c%c%c,",
          atom->name[0], atom->name[1], atom->name[2], atom->name[3]);
  printf(" size %u,", atom->size);
  printf(" current position %u.\n", atom->pos);
#endif
  }
}

void ff_atom(FILE *fh, atom_t *atom, u_int32_t jump)
{
  u_int32_t fwd;

  /* Only move forward until the end of the atom. Probably not really
     going to really fix things if something went wrong enough that
     an atom acquisition's trying to move past the end of an atom, but
     better safe than sorry. */
  fwd = ((atom->pos + jump) < atom->size) ? jump : (atom->size - atom->pos);
  fseek(fh, fwd, SEEK_CUR); /* Move forward in the file */
  atom->pos += fwd; /* Update the atom's position info */
#ifdef DEBUG
  printf("ff_atom: Jumped forward %u bytes.\n", fwd);
  printf("ff_atom: Current position %u.\n", atom->pos);
  printf("ff_atom: Current file position %08X\n", ftell(fh));
#endif
}
