#ifndef ATOMS_H
#define ATOMS_H

#include <sys/types.h>

#define MAXTRACKS 1024 /* Should never need nearly this many */

/* Atom structure typedefs */
/* Note that I only care about a few things that a given atom has.
   If I find I need to care about more, I'll add more, but otherwise,
   the fields I don't use will just get in my way. */
/* I'm also going to probably mostly make up my own fields for the
   udta atom, 'cause that's normally all user-defined and I'm picky
   about having the info I want readily available. */

typedef struct _mvhd_t {
  u_int32_t timescale;
  u_int32_t duration;
} mvhd_t;

typedef struct _tkhd_t {
  u_int32_t track_id;
  u_int32_t duration;
} tkhd_t;

typedef struct _mdhd_t {
  u_int32_t timescale;
  u_int32_t duration;
} mdhd_t;

typedef struct _hdlr_t {
  u_char component_type[4];
  u_char component_subtype[4];
} hdlr_t;

typedef struct _smhd_t {
  int is_audio;
  char balance[2]; /* Fixed-point number */
} smhd_t;

/*
typedef struct _dref_t_t {
  u_int32_t size;
  u_char type[4];
  char *d;
} dref_t_t;

typedef struct _dref_t {
  int entries;
  dref_t_t *t;
} dref_t;

typedef struct _dinf_t {
  dref_t dref;
} dinf_t;
*/

typedef struct _esds_t {
  char got_esds;

  u_int16_t esid;
  u_int8_t priority;

  u_int8_t obj_type;
  u_int8_t stream_info;
  u_int32_t buffersize;
  u_int32_t max_bitrate;
  u_int32_t avg_bitrate;

  u_int16_t dConfLen;
  u_char *decConfig;
  
  u_int8_t SL;
} esds_t;

typedef struct _stsd_t_t {
  u_char format[4];
  u_int16_t data_reference;

  u_int16_t channels;
  u_int16_t samplesize;
  u_int16_t compression_id;
  u_int16_t samplerate[2]; /* Fixed-point number */
  esds_t esds;
} stsd_t_t;

typedef struct _stsd_t {
  u_int32_t entries;
  stsd_t_t *t;
} stsd_t;

typedef struct _stts_t_t {
  u_int32_t count;
  u_int32_t duration;
} stts_t_t;

typedef struct _stts_t {
  u_int32_t entries;
  stts_t_t *t;
} stts_t;

typedef struct _stsc_t_t {
  u_int32_t chunk;
  u_int32_t samples;
  u_int32_t id;
} stsc_t_t;

typedef struct _stsc_t {
  u_int32_t entries;
  stsc_t_t *t;
} stsc_t;

typedef struct _stsz_t_t {
  u_int32_t size;
} stsz_t_t;

typedef struct _stsz_t {
  u_int32_t sample_size;
  u_int32_t entries;
  stsz_t_t *t;
} stsz_t;

typedef struct _stco_t_t {
  u_int32_t offset;
} stco_t_t;

typedef struct _stco_t {
  u_int32_t entries;
  stco_t_t *t;
} stco_t;

typedef struct _stbl_t {
  stsd_t stsd;
  stts_t stts;
  stsc_t stsc;
  stsz_t stsz;
  stco_t stco;
} stbl_t;

typedef struct _minf_t {
  smhd_t smhd;
/*  dinf_t dinf; */
  stbl_t stbl;
} minf_t;

typedef struct _mdia_t {
  mdhd_t mdhd;
  hdlr_t hdlr;
  minf_t minf;
} mdia_t;

typedef struct _trak_t {
  tkhd_t tkhd;
  mdia_t mdia;
} trak_t;

typedef struct _udta_t {
  /* Probably have some rather different stuff here than the original lib */
  char *title;
  char *album;
  char *artist;
  char *composer;
  char *date;
  char *genre;

  u_int16_t tracknum;
  u_int16_t tracks;
  u_int8_t is_comp;
  u_int16_t discnum;
  u_int16_t discs;

  char *comment;
} udta_t;

typedef struct _moov_t {
  int tracks;

  mvhd_t mvhd;
  trak_t *trak[MAXTRACKS];
  udta_t udta;
} moov_t;

#endif /* ATOMS_H */
