#include <mp4lib/mp4.h>

int get_moov(FILE *fh, moov_t *moov)
{
  atom_t atom, child_atom;
  int got_moov = 0;

  do {
    atom_head(fh, &atom);
    if (atomid(atom.name) == atomid("moov")) {
#ifdef DEBUG
      printf("Got moov!\n");
#endif
      got_moov = 1;
    } else {
      skip_atom(fh, &atom);
    }
  } while (!got_moov && !feof(fh));

  if (!got_moov) return 0; /* D'oh! File has no moov atom! */

  do {
    atom_head(fh, &child_atom);

    if (atomid(child_atom.name) == atomid("mvhd")) {
      get_mvhd(fh, &(moov->mvhd), &child_atom);
    } else if (atomid(child_atom.name) == atomid("trak")) {
      trak_t *trak = 0;
#ifdef DEBUG
      printf("Got trak!\n");
#endif
      trak = add_trak(moov);
#ifdef DEBUG
      printf("get_moov: Track %d\n", moov->tracks);
#endif
      get_trak(fh, trak, &child_atom);
    } else if (atomid(child_atom.name) == atomid("udta")) {
      get_udta(fh, &(moov->udta), &child_atom);
    } else {
      skip_atom(fh, &child_atom);
    }
    atom.pos += child_atom.size;
  } while (atom.pos < atom.size);

  skip_atom(fh, &atom);
  return 1;
}

void init_moov(moov_t *moov)
{
  int i;

  init_mvhd(&(moov->mvhd));
  moov->tracks = 0;
  for (i = 0; i < MAXTRACKS; i++) {
    moov->trak[i] = 0;
  }
  init_udta(&(moov->udta));
}

void clear_moov(moov_t *moov)
{
  init_moov(moov);
  while (moov->tracks) {
    delete_trak(moov, moov->trak[moov->tracks - 1]);
  }
  clear_udta(&(moov->udta));
}
