#include <mp4lib/mp4.h>

int is_mp4(const char *filename)
{
  atom_t atom;
  FILE *fh;
  int got_moov = 0;

  if (!(fh = fopen(filename, "rb"))) {
    return 0;
  }

  do {
    if (! atom_head(fh, &atom)) {
      fclose(fh);
      return 0;
    } else if (atomid(atom.name) == atomid("moov")) {
      got_moov = 1;
    }
  } while (!got_moov && !feof(fh));

  fclose(fh);

  if (got_moov) {
    return 1;
  } else {
    return 0;
  }
}

int get_audio_track(moov_t *moov)
{
  int i;

  for (i = 0; i < moov->tracks; i++) {
#ifdef DEBUG
    printf("find_audio_trak: trak %d\n", i);
#endif
    if (moov->trak[i]->mdia.minf.smhd.is_audio) {
#ifdef DEBUG
      printf("find_audio_trak: Got audio trak\n");
#endif
      return i;
    }
  }
  return -1; /* Loop ended. Couldn't find audio track. */
}

u_int32_t time_to_sample(u_int32_t seektime, trak_t *trak)
{
  u_int32_t i;
  u_int32_t count, duration;
  u_int32_t target = seektime;
  u_int32_t sample = 0;

  for (i = 0; i < trak->mdia.minf.stbl.stts.entries; i++) {
    count = trak->mdia.minf.stbl.stts.t[i].count;
    duration = trak->mdia.minf.stbl.stts.t[i].duration;
    while (count) {
      if (target < duration) { /* Are we less than one duration away? */
        return sample; /* Yes. We're done. */
      } else { /* No. There's still too much time left unaccounted. */
        target -= duration;
        sample++;
        count--;
      }
    }
    /* All durations in this tts entry exhausted. Loop to the next entry. */
  }
  return 0;
}

/* More or less lifted from v1 mpeg4ip mp4/quicktime lib chunk_of_sample */
u_int32_t sample_to_chunk(trak_t *trak, u_int32_t sample, u_int32_t *first_sample)
{
  u_int32_t chunk, c1, c2, c1samples, range_samples, total = 0;
  u_int32_t c2entry = 0;
  stsc_t_t *t = trak->mdia.minf.stbl.stsc.t;
  u_int32_t entries = trak->mdia.minf.stbl.stsc.entries;

  c1 = 1;
  c1samples = 0;
  c2entry = 0;

  do {
    c2 = t[c2entry].chunk;
    chunk = c2 - c1;
    range_samples = chunk * c1samples;

    if (sample < (total + range_samples)) {
      break;
    }

    c1samples = t[c2entry].samples;
    c1 = c2;

    if (c2entry < entries) {
      c2entry++;
      total += range_samples;
    }
  } while (c2entry < entries);

  if (c1samples) {
    chunk = ((sample - total) / c1samples) + c1;
  } else {
    chunk = 1;
  }

  *first_sample = total + ((chunk - c1) * c1samples);
  return chunk;
}

u_int32_t chunk_to_offset(trak_t *trak, u_int32_t chunk)
{
  u_int32_t entries = trak->mdia.minf.stbl.stco.entries;
  u_int32_t c = (chunk <= entries) ? chunk : entries;

  return trak->mdia.minf.stbl.stco.t[c - 1].offset;
}

void seek_to_sample(FILE *fh, trak_t *trak, u_int32_t sample)
{
  u_int32_t chunk, offset, first_sample, i;

  chunk = sample_to_chunk(trak, sample, &first_sample);
  offset = chunk_to_offset(trak, chunk);
  for (i = first_sample; i < sample; i++) {
    offset += trak->mdia.minf.stbl.stsz.t[i].size;
  }
  fseek(fh, offset, SEEK_SET);
}
