#ifndef MP4LIB_H
#define MP4LIB_H

/* Pull in other defines */
#if HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <sys/types.h>

#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#elif HAVE_STRINGS_H
#include <strings.h>
#endif

#include <mp4lib/atoms.h>

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#define atomid(a) str2int(a, 4)

#if !HAVE_STRCHR
#define strchr index
#endif

#if !HAVE_BZERO && HAVE_MEMSET
#define bzero(buf, bytes) ((void) memset(buf, 0, bytes))
#endif

/* data structures */

typedef struct _atom_t {
  u_int64_t size;
  u_int64_t pos;
  unsigned char name[5];
} atom_t;

/* Function prototypes */

/* mp4.c */
int is_mp4(const char *);
int get_audio_track(moov_t *);
u_int32_t time_to_sample(u_int32_t, trak_t *);
void seek_to_sample(FILE *, trak_t *, u_int32_t);

/* atom.c */
int atom_head(FILE *, atom_t *);
void skip_atom(FILE *, atom_t *);

/* data.c */
u_int64_t str2int(const unsigned char *, int);

/* moov.c */
int get_moov(FILE *, moov_t *);
void init_moov(moov_t *);
void clear_moov(moov_t *);

/* mvhd.c */
int get_mvhd(FILE *, mvhd_t *, atom_t *);
void init_mvhd(mvhd_t *);

/* trak.c */
int get_trak(FILE *, trak_t *, atom_t *);
trak_t *add_trak(moov_t *);
void init_trak(trak_t *);
int delete_trak(moov_t *, trak_t *);

/* tkhd.c */
int get_tkhd(FILE *, tkhd_t *, atom_t *);
void init_tkhd(tkhd_t *);

/* mdia.c */
int get_mdia(FILE *, mdia_t *, atom_t *);
void init_mdia(mdia_t *);
void clear_mdia(mdia_t *);

/* mdhd.c */
int get_mdhd(FILE *, mdhd_t *, atom_t *);
void init_mdhd(mdhd_t *);

/* hdlr.c */
int get_hdlr(FILE *, hdlr_t *, atom_t *);
void init_hdlr(hdlr_t *);

/* minf.c */
int get_minf(FILE *, minf_t *, atom_t *);
void init_minf(minf_t *);
void clear_minf(minf_t *);

/* smhd.c */
int get_smhd(FILE *, smhd_t *, atom_t *);
void init_smhd(smhd_t *);

/* stbl.c */
int get_stbl(FILE *, stbl_t *, atom_t *);
void init_stbl(stbl_t *);
void clear_stbl(stbl_t *);

/* stsd.c */
int get_stsd(FILE *, stsd_t *, atom_t *);
void init_stsd(stsd_t *);
void clear_stsd(stsd_t *);

/* esds.c */
int get_esds(FILE *, esds_t *, atom_t *);
void init_esds(esds_t *);
void clear_esds(esds_t *);

/* stts.c */
int get_stts(FILE *, stts_t *, atom_t *);
void init_stts(stts_t *);
void clear_stts(stts_t *);

/* stsc.c */
int get_stsc(FILE *, stsc_t *, atom_t *);
void init_stsc(stsc_t *);
void clear_stsc(stsc_t *);

/* stsz.c */
int get_stsz(FILE *, stsz_t *, atom_t *);
void init_stsz(stsz_t *);
void clear_stsz(stsz_t *);

/* stco.c */
int get_stco(FILE *, stco_t *, atom_t *);
void init_stco(stco_t *);
void clear_stco(stco_t *);

/* udta.c */
int get_udta(FILE *, udta_t *, atom_t *);
void init_udta(udta_t *);
void clear_udta(udta_t *);

#endif /* MP4LIB_H */
