#include <mp4lib/mp4.h>

void init_stsd_t(stsd_t_t *t)
{
  bzero(t->format, 4);
  t->data_reference = 0;
  t->channels = 0;
  t->samplesize = 0;
  t->compression_id = 0;
  bzero(t->samplerate, 4);
  init_esds(&(t->esds));
}

void clear_stsd_t(stsd_t_t *t)
{
  bzero(t->format, 4);
  t->data_reference = 0;
  t->channels = 0;
  t->samplesize = 0;
  t->compression_id = 0;
  bzero(t->samplerate, 4);
  clear_esds(&(t->esds));
}

int get_stsd_t(FILE *fh, stsd_t_t *t, atom_t *atom)
{
  unsigned char Buf[4];
  atom_t subatom;

  atom_head(fh, &subatom);
  memcpy(t->format, subatom.name, 4); /* Get name of audio format */
  ff_atom(fh, &subatom, 6); /* Skip reserved bytes */
  if ((fread(Buf, 1, 2, fh)) < 2) {
    t->data_reference = 0;
    return 0;
  }
  t->data_reference = str2int(Buf, 2);
  subatom.pos += 2;
  ff_atom(fh, &subatom, 8); /* Skip version, revision, vendor */
  if ((fread(Buf, 1, 2, fh)) < 2) {
    t->channels = 0;
    return 0;
  }
  t->channels = str2int(Buf, 2);
  subatom.pos += 2;
  if ((fread(Buf, 1, 2, fh)) < 2) {
    t->samplesize = 0;
    return 0;
  }
  t->samplesize = str2int(Buf, 2);
  subatom.pos += 2;
  if ((fread(Buf, 1, 2, fh)) < 2) {
    t->compression_id = 0;
    return 0;
  }
  t->compression_id = str2int(Buf, 2);
  subatom.pos += 2;
  ff_atom(fh, &subatom, 2); /* Skip the packet size */
  if ((fread(Buf, 1, 4, fh)) < 4) {
    bzero(t->samplerate, 4);
    return 0;
  }
  t->samplerate[0] = str2int(Buf, 2);
  t->samplerate[1] = str2int(Buf + 2, 2);
  subatom.pos += 4;

  /* Look for the esds stuff. Ahgahd. */
  while (subatom.pos < subatom.size) {
    atom_t quark;

    atom_head(fh, &quark);
    if (atomid(quark.name) == atomid("esds")) {
      get_esds(fh, &(t->esds), &quark);
      skip_atom(fh, &quark);
    } else {
      skip_atom(fh, &quark);
    }
    subatom.pos += quark.size;
  }
  skip_atom(fh, &subatom);
  atom->pos += subatom.size;
}

int get_stsd(FILE *fh, stsd_t *stsd, atom_t *atom)
{
  unsigned char Buf[4];
  u_int32_t i;

#ifdef DEBUG
  printf("Current file position %08X\n", ftell(fh));
#endif
  ff_atom(fh, atom, 4); /* Skip version, flags */
#ifdef DEBUG
  printf("Current file position %08X\n", ftell(fh));
#endif
  if ((fread(Buf, 1, 4, fh)) < 4) {
    stsd->entries = 0;
    return 0;
  }
  stsd->entries = str2int(Buf, 4);
  atom->pos += 4;
  stsd->t = (stsd_t_t *) malloc(sizeof(stsd_t_t) * stsd->entries);
  for (i = 0; i < stsd->entries; i++) {
    init_stsd_t(&(stsd->t[i]));
    get_stsd_t(fh, &(stsd->t[i]), atom);
  }

  skip_atom(fh, atom);
#ifdef DEBUG
  printf("Current file position %08X\n", ftell(fh));
#endif
  return 1;
}

void init_stsd(stsd_t *stsd)
{
  stsd->entries = 0;
  stsd->t = NULL;
}

void clear_stsd(stsd_t *stsd)
{
  if (stsd->entries) {
    u_int32_t i;

    for (i = 0; i < stsd->entries; i++) {
      clear_stsd_t(&(stsd->t[i]));
    }
    free(stsd->t);
    stsd->entries = 0;
  }
}
