#include <mp4lib/mp4.h>

void init_stsz_t(stsz_t_t *t)
{
  t->size = 0;
}

int get_stsz_t(FILE *fh, stsz_t_t *t, atom_t *atom)
{
  unsigned char Buf[4];

  if ((fread(Buf, 1, 4, fh)) < 4) {
    t->size = 0;
    return 0;
  }
  t->size = str2int(Buf, 4);
  atom->pos += 4;
}

int get_stsz(FILE *fh, stsz_t *stsz, atom_t *atom)
{
  unsigned char Buf[4];
  u_int32_t i;

#ifdef DEBUG
  printf("Current file position %08X\n", ftell(fh));
#endif
  ff_atom(fh, atom, 4); /* Skip version, flags */
  if ((fread(Buf, 1, 4, fh)) < 4) {
    stsz->sample_size = 0;
    return 0;
  }
  stsz->sample_size = str2int(Buf, 4);
  atom->pos += 4;
  if (stsz->sample_size) { /* All samples have same size, no need to
                              get per-sample sizes */
    skip_atom(fh, atom);
    return 1;
  }
  /* Okay. They're not all the same size, so get per-sample sizes. */
  if ((fread(Buf, 1, 4, fh)) < 4) {
    stsz->entries = 0;
    return 0;
  }
  stsz->entries = str2int(Buf, 4);
  atom->pos += 4;
  stsz->t = (stsz_t_t *) malloc(sizeof(stsz_t_t) * stsz->entries);
  for (i = 0; i < stsz->entries; i++) {
    init_stsz_t(&(stsz->t[i]));
    get_stsz_t(fh, &(stsz->t[i]), atom);
  }

  skip_atom(fh, atom);
#ifdef DEBUG
  printf("Current file position %08X\n", ftell(fh));
#endif
  return 1;
}

void init_stsz(stsz_t *stsz)
{
  stsz->sample_size = 0;
  stsz->entries = 0;
  stsz->t = NULL;
}

void clear_stsz(stsz_t *stsz)
{
  stsz->sample_size = 0;
  if (stsz->entries) {
    free(stsz->t);
  }
  stsz->entries = 0;
  stsz->t = NULL;
}
