#include "mp4.h"

int main(int argc, char *argv[])
{
  FILE *m4a_fh = NULL;
  moov_t moov;
  int i, k;
  char *comment = NULL;

  if (! (argc > 1)) {
    printf("Usage: %s <file-to-parse>\n", argv[0]);
    return 0;
  } else {
/*    if (is_mp4(argv[1])) {
      printf("Is an mp4.\n");
    } */
    m4a_fh = fopen(argv[1], "rb");
    if (! m4a_fh) {
      printf("Unable to open file %s\n", argv[1]);
      return -1;
    }
    init_moov(&moov);
    printf("Beginning parse run on %s\n", argv[1]);
    if (get_moov(m4a_fh, &moov)) {
      printf("Found moov atom.\n");
    }
    printf("Timescale %u\n", moov.mvhd.timescale);
    printf("Duration %u", moov.mvhd.duration);
    printf(" (%u seconds)\n", (moov.mvhd.duration / moov.mvhd.timescale));
    printf("%u Tracks:\n", moov.tracks);
    for (i = 0; i < moov.tracks; i++) {
      unsigned char Buf[5];

      printf("Track %d, ", i);
      printf("ID %u, ", moov.trak[i]->tkhd.track_id);
      printf(" tkhd duration %u.\n", moov.trak[i]->tkhd.duration);
      printf("  mdhd timescale %u,", moov.trak[i]->mdia.mdhd.timescale);
      printf(" duration %u\n", moov.trak[i]->mdia.mdhd.duration);
      memcpy(Buf, moov.trak[i]->mdia.hdlr.component_subtype, 4);
      Buf[4] = 0;
      printf("  hdlr subtype %s\n", Buf);
      if (moov.trak[i]->mdia.minf.smhd.is_audio) {
        printf("  smhd says is audio, balance %d.%d\n",
               moov.trak[i]->mdia.minf.smhd.balance[0],
               moov.trak[i]->mdia.minf.smhd.balance[1]);
      }
      if (moov.trak[i]->mdia.minf.stbl.stsd.entries) {
        u_int32_t j;

        for (j = 0; j < moov.trak[i]->mdia.minf.stbl.stsd.entries; j++) {
          stsd_t_t t = moov.trak[i]->mdia.minf.stbl.stsd.t[j];
          memcpy(Buf, t.format, 4);
          Buf[4] = 0;
          printf("  Data format %s\n", Buf);
          printf("  Sample description ID %u\n", t.data_reference);
          printf("  %u channels,", t.channels);
          printf(" %u bits,", t.samplesize);
          printf(" compression ID %u,", t.compression_id);
          printf(" sample rate %d.%d", t.samplerate[0], t.samplerate[1]);
          printf("\n");
          if (t.esds.got_esds) {
            printf("  Got ESDS.\n");
            printf("   ESID %u,", t.esds.esid);
            printf(" Priority %u\n", t.esds.priority);
            printf("   Object type %02X,", t.esds.obj_type);
            printf(" streaming %d,", t.esds.stream_info);
            printf(" buffer size %u\n", t.esds.buffersize);
            printf("   Max bit rate %u,", t.esds.max_bitrate);
            printf(" Avg bit rate %u\n", t.esds.avg_bitrate);
            if (t.esds.dConfLen) {
              printf("   Decoder config: ");
              for (k = 0; k < t.esds.dConfLen; k++) {
                printf("%02X", t.esds.decConfig[k]);
              }
              printf("\n");
            }
          }
          printf("  %u time-to-sample entries:\n",
                    moov.trak[i]->mdia.minf.stbl.stts.entries);
          for (k = 0; k < moov.trak[i]->mdia.minf.stbl.stts.entries; k++) {
            stts_t_t t = moov.trak[i]->mdia.minf.stbl.stts.t[k];
            printf("   %u samples", t.count);
            printf(" have duration %u\n", t.duration);
          }
          printf("  %u sample-to-chunk entries:\n",
                    moov.trak[i]->mdia.minf.stbl.stsc.entries);
          for (k = 0; k < moov.trak[i]->mdia.minf.stbl.stsc.entries; k++) {
            stsc_t_t t = moov.trak[i]->mdia.minf.stbl.stsc.t[k];
            printf("   First chunk %i,", t.chunk);
            printf(" %u samples per chunk,", t.samples);
            printf(" Sample desc ID %u\n", t.id);
          }
          if (moov.trak[i]->mdia.minf.stbl.stsz.sample_size) {
            printf("  All samples have size %u.\n",
                        moov.trak[i]->mdia.minf.stbl.stsz.sample_size);
          } else {
            printf("  %u sample sizes:\n",
                      moov.trak[i]->mdia.minf.stbl.stsz.entries);
            for (k = 0; k < moov.trak[i]->mdia.minf.stbl.stsz.entries; k++) {
              stsz_t_t t = moov.trak[i]->mdia.minf.stbl.stsz.t[k];
              printf("   Sample %u,", k);
              printf(" sample size %u\n", t.size);
            }
          }
          printf("  %u chunk offset entries:\n",
                    moov.trak[i]->mdia.minf.stbl.stco.entries);
          for (k = 0; k < moov.trak[i]->mdia.minf.stbl.stco.entries; k++) {
            stco_t_t t = moov.trak[i]->mdia.minf.stbl.stco.t[k];
            printf("   Chunk %u,", k + 1);
            printf(" Offset %u\n", t.offset);
          }
        }
      }
    }
    if (moov.udta.title) {
      printf("Title: '%s'\n", moov.udta.title);
    }
    if (moov.udta.album) {
      printf("Album: '%s'\n", moov.udta.album);
    }
    if (moov.udta.artist) {
      printf("Performed by '%s'\n", moov.udta.artist);
    }
    if (moov.udta.composer) {
      printf("Composed by '%s'\n", moov.udta.composer);
    }
    if (moov.udta.date) {
      printf("Created %s.\n", moov.udta.date);
    }
    if (moov.udta.genre) {
      printf("Genre: '%s'\n", moov.udta.genre);
    }
    if (moov.udta.tracknum) {
      printf("Track Number %u", moov.udta.tracknum);
      if (moov.udta.tracks) {
        printf(" of %u tracks.\n", moov.udta.tracks);
      }
    } else if (moov.udta.tracks) {
      printf("%u tracks total.\n", moov.udta.tracks);
    }
    if (moov.udta.discnum) {
      printf("Disc Number %u", moov.udta.discnum);
      if (moov.udta.discs) {
        printf(" of %u.\n", moov.udta.discs);
      }
    } else if (moov.udta.discs) {
      printf("%u discs total.\n", moov.udta.discs);
    }
    if (moov.udta.is_comp) {
      printf("(Compilation)\n");
    }
    if (moov.udta.comment) {
      printf("Comments: %s\n", moov.udta.comment);
    }
    /* printf("%s\n", "\xa9""alb"); */
    printf("End Of File reached.\n");
    fclose(m4a_fh);
  }
}
