#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

const char *Containers[] = {
                            "moov", "trak", "mdia", "minf", "dinf", "stbl",
                            "udta", "meta", "ilst", "alb", "----"
                           };
u_int64_t str2int(const unsigned char *Buf, int Bytes)
{ 
  u_int64_t Length = 0;
  int Bcount;

#ifdef DEBUG
  printf("str2int: ");
#endif
  for (Bcount = 0; Bcount < (Bytes - 1); Bcount++) {
#ifdef DEBUG
    printf("%x ", Buf[Bcount]);
#endif
    Length += Buf[Bcount];
    Length <<= 8;
  }
#ifdef DEBUG
  printf("%x\n", Buf[Bytes - 1]);
#endif
  Length += Buf[Bytes - 1];
  return Length;
}

#define atomid(a) str2int(a, 4)

char isContainer(char *atomType)
{
  char atomIndex;

  for (atomIndex = 0; atomIndex < (sizeof(Containers) / 4); atomIndex++) {
    if (!strncmp(atomType, Containers[atomIndex], 4)) {
      return 1;
    }
  }
  return 0;
}

char *indent(char inLevel)
{
  char *spcBuf = 0;

  if (!(spcBuf = malloc((sizeof(char) * inLevel) + 1))) {
    printf("Unable to malloc space for indents.\n");
    exit(1);
  }
  memset(spcBuf, ' ', inLevel);
  spcBuf[inLevel] = '\0';
  return spcBuf;
}

#define TIME_ADJ ((u_int64_t) 70 * 365 * 24 * 60 * 60)

typedef union {
  struct {
    u_int8_t Version;
    u_int8_t Flags[3];
    u_int32_t CreateTime;
    u_int32_t ModTime;
    u_int32_t TimeScale;
    u_int32_t Duration;
    u_int32_t PrefRate;
    u_int16_t PrefVolume;
    u_int8_t Reserved[10];
    u_int8_t Matrix[36];
    u_int32_t PreviewTime;
    u_int32_t PreviewDuration;
    u_int32_t PosterTime;
    u_int32_t SelectionTime;
    u_int32_t SelectionDuration;
    u_int32_t CurrentTime;
    u_int32_t NextTrackID;
  } s;
  u_int8_t d[100];
} mvhd;

typedef union {
  struct {
    u_int8_t Version;
    u_int8_t Flags[3];
    u_int32_t CreateTime;
    u_int32_t ModTime;
    u_int32_t TrackID;
    u_int8_t Reserved1[4];
    u_int32_t Duration;
    u_int8_t Reserved2[4];
    u_int16_t Layer;
    u_int16_t AltGroup;
    u_int16_t Volume;
    u_int8_t Reserved3[2];
    u_int8_t Matrix[36];
    u_int32_t TrackWidth;
    u_int32_t TrackHeight;
  } s;
  u_int8_t d[84];
} tkhd;

typedef union {
  struct {
    u_int8_t Version;
    u_int8_t Flags[3];
    u_int32_t CreateTime;
    u_int32_t ModTime;
    u_int32_t TimeScale;
    u_int32_t Duration;
    u_int16_t Language;
    u_int16_t Quality;
  } s;
  u_int8_t d[24];
} mdhd;

void processData(FILE *fh, u_int64_t dataSize, char *prespace, char *atomType)
{
  unsigned char *atomData;
  int NshowPacket;
  int i, j;
  u_int64_t typeID;

  if (! (atomData = malloc(sizeof(char) * dataSize))) {
    printf("Unable to malloc space for atom data.\n");
    exit(1);
  }

  if (fread(atomData, 1, dataSize, fh) < dataSize) {
    printf("Unable to read all of atom data.\n");
    exit(1);
  }

  typeID = atomid(atomType);
  if (typeID == atomid("mvhd")) {
    mvhd a_mvhd;

    memcpy(a_mvhd.d, atomData, sizeof(mvhd));
    printf("%s Time Scale: %u\n", prespace, ntohl(a_mvhd.s.TimeScale));
    printf("%s Duration: %u units (%f seconds)\n", prespace,
           ntohl(a_mvhd.s.Duration),
           ((float) ntohl(a_mvhd.s.Duration) /
                    ntohl(a_mvhd.s.TimeScale)));
    printf("%s Preferred rate: %u\n", prespace, (a_mvhd.s.PrefRate));
    printf("%s Current time: %u\n", prespace, ntohl(a_mvhd.s.CurrentTime));
    printf("%s Next track ID: %u\n", prespace, ntohl(a_mvhd.s.NextTrackID));
  } else if (typeID == atomid("tkhd")) {
    tkhd a_tkhd;
    memcpy(a_tkhd.d, atomData, sizeof(tkhd));

    printf("%s Track ID: %u\n", prespace, ntohl(a_tkhd.s.TrackID));
    printf("%s Duration: %u\n", prespace, ntohl(a_tkhd.s.Duration));
  } else if (typeID == atomid("mdhd")) {
    mdhd a_mdhd;
    memcpy(a_mdhd.d, atomData, sizeof(mdhd));

    printf("%s Time Scale: %u\n", prespace, ntohl(a_mdhd.s.TimeScale));
    printf("%s Duration: %u\n", prespace, ntohl(a_mdhd.s.Duration));
    printf("%s Language: %u (%04X)\n", prespace, ntohs(a_mdhd.s.Language),
                                                 ntohs(a_mdhd.s.Language));
    printf("%s Quality: %u\n", prespace, ntohs(a_mdhd.s.Quality));
  } else if (typeID == atomid("stsd")) {
    u_int32_t descNum, dCount, dOff, dSize;
    char dName[5];
    u_int16_t drIndex;

    descNum = str2int(atomData + 4, 4);
    printf("%s %u Entries:\n", prespace, descNum);
    for (dCount = 0, dOff = 8; dCount < descNum; dCount++) {
      dSize = str2int(atomData + dOff, 4);
      printf("%s  Entry size %u.\n", prespace, dSize);
      dOff += 4;
      printf("%s  Data format %08X", prespace, str2int(atomData + dOff, 4));
      memcpy(dName, atomData + dOff, 4);
      dName[4] = '\0';
      printf(" (%s).\n", dName);
      dOff += 4;
      dOff += 6; /* Skip over 6 reserved bytes. */
      drIndex = str2int(atomData + dOff, 2);
      printf("%s  Sample description ID: %u.\n", prespace, drIndex);
      dOff += 2;
      printf("%s  Version: %u\n", prespace, str2int(atomData + dOff, 2));
      dOff += 2;
      dOff += 6; /* Skip Revision level and Vendor. */
      printf("%s  %u Channels.\n", prespace, str2int(atomData + dOff, 2));
      dOff += 2;
      printf("%s  %u Bits.\n", prespace, str2int(atomData + dOff, 2));
      dOff += 2;
      printf("%s  Compression ID %u.\n", prespace, str2int(atomData + dOff, 2));
      dOff += 2;
      dOff += 2; /* Skip Packet size. */
      printf("%s  Sample rate %u.%u\n", prespace,
                                        str2int(atomData + dOff, 2),
                                        str2int(atomData + dOff + 2, 2));
      dOff += 4;
      dOff += 25; /* Skip ESDS head crap. */
      printf("%s  Audio type %u.\n", prespace, str2int(atomData + dOff, 1));
      dOff++;
      dOff++; /* Bit fields?! Ew. */
      printf("%s  Buffer size: %u\n", prespace, str2int(atomData + dOff, 3));
      dOff += 3;
      printf("%s  Max bit rate: %u\n", prespace, str2int(atomData + dOff, 4));
      dOff += 4;
      printf("%s  Avg. bit rate: %u\n", prespace, str2int(atomData + dOff, 4));
      dOff += 4;

      printf("%s  (dOff = %u)\n", prespace, dOff);
    }
  } else if (typeID == atomid("stts")) {
    u_int32_t sttsNum;

    sttsNum = str2int(atomData + 4, 4);
    printf("%s %u Entries:\n", prespace, sttsNum);
    for (i = 8; i < dataSize; i += 8) {
      printf("%s  %u samples ", prespace, str2int(atomData + i, 4));
      printf("have duration %u bytes\n", str2int(atomData + i + 4, 4));
    }
  } else if (typeID == atomid("stsc")) {
    u_int32_t stscNum;

    stscNum = str2int(atomData + 4, 4);
    printf("%s %u Entries:\n", prespace, stscNum);
    for (i = 8; i < dataSize; i += 12) {
      printf("%s  First chunk %u", prespace, str2int(atomData + i, 4));
      printf(", %u samples per chunk", str2int(atomData + i + 4, 4));
      printf(", Sample decription ID %u\n", str2int(atomData + i + 8, 4));
    }
  } else if (typeID == atomid("stsz")) {
    u_int32_t Samples, sCount;

    Samples = str2int(atomData + 4, 4);
    if (Samples > 0) {
      printf("%s Sample size %u.\n", prespace, Samples);
    } else {
      Samples = str2int(atomData + 8, 4);
      printf("%s %u sample sizes:\n", prespace, Samples);
      for (sCount = 1; sCount <= Samples; sCount++) {
        printf("%s  Sample %u, ", prespace, sCount);
        printf("sample size %u\n", str2int(atomData + 8 + (4 * sCount), 4));
      }
    }
  } else if (typeID == atomid("stco")) {
    u_int32_t Offsets, oCount;

    Offsets = str2int(atomData + 4, 4);
    printf("%s %u Chunk offsets:\n", prespace, Offsets - 1);
    for (oCount = 1; oCount < Offsets; oCount++) {
      printf("%s  Chunk %u, Offset %u\n", prespace, oCount,
                            str2int(atomData + 8 + (4 * oCount), 4));
    }
  } else {
    if (dataSize > 255) {
      NshowPacket = 255;
    } else {
      NshowPacket = dataSize;
    }

    printf("%sInitial packet contents:\n", prespace);

    for (i = 0; i < NshowPacket; i += 16) {
      printf("%s", prespace);
      for (j = 0; j < 16; j += 2) {
        if ((i+j) < NshowPacket) {
          printf("%02X", atomData[i+j]);
        } else {
          printf("  ");
        }
        if ((i+j+1) < NshowPacket) {
          printf("%02X", atomData[i+j+1]);
        } else {
          printf("  ");
        }
        printf(" ");
      }
      for (j = 0; (j < 16) && (i + j) < NshowPacket; j++) {
        if ((atomData[i+j] >= 0x20) && (atomData[i+j] <= 0x7E)) {
          printf("%c", atomData[i+j]);
        } else {
          printf(".");
        }
      }
      printf("\n");
    }
  }

  free(atomData);
}

void ParseFile(FILE *fh, char isRoot, u_int64_t maxLength, char iLvl)
{
  u_int64_t soFar = 0;
  char hdrCount;
  unsigned char atomSizeBuf[8], atomTypeBuf[5];
  u_int64_t atomSize;
  char *prespace;
  int sizeRead;

  while ((isRoot && !feof(fh)) || (!isRoot && (soFar < maxLength))) {
    hdrCount = 0;
    sizeRead = fread(atomSizeBuf, 1, 4, fh);
    if (sizeRead < 4) {
      if ((sizeRead == 0) && (feof(fh))) { break; }
      printf("Couldn't read full atom length.\n");
      exit(1);
    }
    atomSize = str2int(atomSizeBuf, 4);
    hdrCount += 4;
    /* Compensate for old QuickTime stupid udta atom bug */
    if ((isRoot && feof(fh)) ||
        (!isRoot && ((soFar + 4) >= maxLength)) ||
        (! atomSize)) {
      break;
    }

    if (fread(atomTypeBuf, 1, 4, fh) < 4) {
      printf("Couldn't read atom type.\n");
      exit(1);
    }
    atomTypeBuf[4] = '\0';
    hdrCount += 4;

    /* If atomSize is 1, there's a 64-bit extended size after the atom type. */
    if (atomSize == 1) {
      if (fread(atomSizeBuf, 1, 8, fh) < 8) {
        printf("Couldn't read extended atom length.\n");
        exit(1);
      }
      atomSize = str2int(atomSizeBuf, 8);
      hdrCount += 8;
    }

    /* Get the # of spaces to indent by. */
    prespace = indent(iLvl);
    /* Is it a container atom, or a data atom? Handle accordingly. */
    if (isContainer(atomTypeBuf)) {
      printf("%sContainer atom type %s, header length %d, length %u.", 
             prespace, atomTypeBuf, hdrCount, atomSize);
      printf(" Contains:\n", prespace);
/*      fseek(fh, atomSize - hdrCount, SEEK_CUR); */
      if (!strncmp(atomTypeBuf, "meta", 4)) {
        fseek(fh, 4, SEEK_CUR);
        hdrCount += 4;
      }
      ParseFile(fh, FALSE, atomSize - hdrCount, iLvl + 1);
    } else {
      printf("%sData atom type %s, length %u.\n",
             prespace, atomTypeBuf, atomSize);
/*      fseek(fh, atomSize - hdrCount, SEEK_CUR); */
      processData(fh, atomSize - hdrCount, prespace, atomTypeBuf);
    }
    free(prespace);

    /* Jump counter to end of atom. */
    soFar += atomSize;
    /* More QT 1.0 stupid udta bug work-around. Jump 4 0's at end of atom. */
    if (!strncmp(atomTypeBuf, "udta", 4)) {
      soFar += 4;
    }
  }
}

int main(int argc, char *argv[])
{
  FILE *m4a_fh = NULL;

  if (! (argc > 1)) {
    printf("Usage: %s <file-to-parse>\n", argv[0]);
    return 0;
  } else {
    m4a_fh = fopen(argv[1], "rb");
    if (! m4a_fh) {
      printf("Unable to open file %s\n", argv[1]);
      return -1;
    }
    printf("Beginning parse run on %s\n", argv[1]);
    ParseFile(m4a_fh, TRUE, NULL, 0);
    printf("End Of File reached.\n");
    fclose(m4a_fh);
  }
}
