/*
 * File info box for libm4a plugin
 * by Chris Pinard <slarti@gweep.net>, 25 Feb 2004
 */
#include <src/libm4a.h>

static GtkWidget *window = NULL;

static GtkWidget *filename_entry;

static GtkWidget *song_frame;
static GtkWidget *title_entry, *album_entry, *artist_entry, *composer_entry;
static GtkWidget *track_info, *year_entry;
static GtkWidget *disc_info, *is_comp_entry;
static GtkWidget *genre_entry, *comment_entry;
static GtkWidget *title_label, *album_label, *artist_label, *composer_label;
static GtkWidget *year_label;
static GtkWidget *is_comp_label;
static GtkWidget *genre_label, *comment_label;

static GtkWidget *file_frame;
static GtkWidget *file_bitrate, *file_samplerate;
static GtkWidget *file_channels, *file_samples;

extern player_t *m4a_info;

void m4a_info_box(char *filename)
{
  char *tmp, *title;
  moov_t *moov;

  if (!window)
  {
    GtkWidget *all_vbox, *label;
    GtkWidget *filename_hbox, *info_hbox, *ok_hbox;
    GtkWidget *table;
    GtkWidget *disc_hbox;
    GtkWidget *file_box;
    GtkWidget *ok_but;
    
    window = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
    gtk_signal_connect(GTK_OBJECT(window), "destroy",
           gtk_widget_destroyed, &window);
    gtk_container_set_border_width(GTK_CONTAINER(window), 10);

    all_vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_add(GTK_CONTAINER(window), all_vbox);

    filename_hbox = gtk_hbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(all_vbox), filename_hbox, FALSE, TRUE, 0);

    label = gtk_label_new("Filename:");
    gtk_box_pack_start(GTK_BOX(filename_hbox), label, FALSE, TRUE, 0);
    filename_entry = gtk_entry_new();
    gtk_editable_set_editable(GTK_EDITABLE(filename_entry), FALSE);
    gtk_box_pack_start(GTK_BOX(filename_hbox), filename_entry, TRUE, TRUE, 0);
    
    info_hbox = gtk_hbox_new(FALSE, 10);
    gtk_box_pack_start(GTK_BOX(all_vbox), info_hbox, FALSE, TRUE, 0);
    
    song_frame = gtk_frame_new("Song Info:");
    gtk_box_pack_start(GTK_BOX(info_hbox), song_frame, FALSE, FALSE, 0);

    table = gtk_table_new(6, 13, FALSE);
    gtk_container_set_border_width(GTK_CONTAINER(table), 5);
    gtk_container_add(GTK_CONTAINER(song_frame), table);

    title_label = gtk_label_new("Title:");
    gtk_misc_set_alignment(GTK_MISC(title_label), 1, 0.5);
    gtk_table_attach(GTK_TABLE(table), title_label, 0, 1, 0, 1,
         GTK_FILL, GTK_FILL, 5, 5);
    title_entry = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(title_entry), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), title_entry, 1, 6, 0, 1,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);

    album_label = gtk_label_new("Album:");
    gtk_misc_set_alignment(GTK_MISC(album_label), 1, 0.5);
    gtk_table_attach(GTK_TABLE(table), album_label, 0, 1, 1, 2,
         GTK_FILL, GTK_FILL, 5, 5);
    album_entry = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(album_entry), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), album_entry, 1, 6, 1, 2,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);

    artist_label = gtk_label_new("Artist:");
    gtk_misc_set_alignment(GTK_MISC(artist_label), 1, 0.5);
    gtk_table_attach(GTK_TABLE(table), artist_label, 0, 1, 2, 3,
         GTK_FILL, GTK_FILL, 5, 5);
    artist_entry = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(artist_entry), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), artist_entry, 1, 6, 2, 3,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);

    composer_label = gtk_label_new("Composer:");
    gtk_misc_set_alignment(GTK_MISC(composer_label), 1, 0.5);
    gtk_table_attach(GTK_TABLE(table), composer_label, 0, 1, 3, 4,
         GTK_FILL, GTK_FILL, 5, 5);
    composer_entry = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(composer_entry), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), composer_entry, 1, 6, 3, 4,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);

    track_info = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(track_info), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), track_info, 1, 4, 4, 5,
           GTK_FILL | GTK_EXPAND | GTK_SHRINK,
           GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);

    year_label = gtk_label_new("Year:");
    gtk_misc_set_alignment(GTK_MISC(year_label), 1, 0.5);
    gtk_table_attach(GTK_TABLE(table), year_label, 4, 5, 4, 5,
         GTK_FILL, GTK_FILL, 5, 5);
    year_entry = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(year_entry), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), year_entry, 5, 6, 4, 5,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);

    disc_info = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(disc_info), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), disc_info, 1, 4, 5, 6,
           GTK_FILL | GTK_EXPAND | GTK_SHRINK,
           GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);

    is_comp_entry = gtk_label_new("(Compilation)");
    gtk_misc_set_alignment(GTK_MISC(is_comp_entry), 0.5, 0.5);
    gtk_table_attach(GTK_TABLE(table), is_comp_entry, 4, 6, 5, 6,
         GTK_FILL, GTK_FILL, 5, 5);
    gtk_widget_hide(GTK_WIDGET(is_comp_entry));

    genre_label = gtk_label_new("Genre:");
    gtk_misc_set_alignment(GTK_MISC(genre_label), 1, 0.5);
    gtk_table_attach(GTK_TABLE(table), genre_label, 0, 1, 6, 7,
         GTK_FILL, GTK_FILL, 5, 5);
    genre_entry = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(genre_entry), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), genre_entry, 1, 6, 6, 7,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);

    comment_label = gtk_label_new("Comment:");
    gtk_misc_set_alignment(GTK_MISC(comment_label), 1, 0.5);
    gtk_table_attach(GTK_TABLE(table), comment_label, 0, 1, 7, 8,
         GTK_FILL, GTK_FILL, 5, 5);
    comment_entry = gtk_text_new(NULL, NULL);
    gtk_text_set_editable(GTK_TEXT(comment_entry), FALSE);
    gtk_text_set_word_wrap(GTK_TEXT(comment_entry), TRUE);
    gtk_table_attach(GTK_TABLE(table), comment_entry, 1, 6, 7, 13,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK,
         GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);

    file_frame = gtk_frame_new("File Info:");
    gtk_box_pack_start(GTK_BOX(info_hbox), file_frame, FALSE, FALSE, 0);

    file_box = gtk_vbox_new(FALSE, 5);
    gtk_container_add(GTK_CONTAINER(file_frame), file_box);
    gtk_container_set_border_width(GTK_CONTAINER(file_box), 10);
    gtk_box_set_spacing(GTK_BOX(file_box), 0);

    file_bitrate = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(file_bitrate), 0, 0);
    gtk_label_set_justify(GTK_LABEL(file_bitrate), GTK_JUSTIFY_LEFT);
    gtk_box_pack_start(GTK_BOX(file_box), file_bitrate, FALSE, FALSE, 0);

    file_samplerate = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(file_samplerate), 0, 0);
    gtk_label_set_justify(GTK_LABEL(file_samplerate), GTK_JUSTIFY_LEFT);
    gtk_box_pack_start(GTK_BOX(file_box), file_samplerate, FALSE, FALSE, 0);

    file_channels = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(file_channels), 0, 0);
    gtk_label_set_justify(GTK_LABEL(file_channels), GTK_JUSTIFY_LEFT);
    gtk_box_pack_start(GTK_BOX(file_box), file_channels, FALSE, FALSE, 0);

    file_samples = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(file_samples), 0, 0);
    gtk_label_set_justify(GTK_LABEL(file_samples), GTK_JUSTIFY_LEFT);
    gtk_box_pack_start(GTK_BOX(file_box), file_samples, FALSE, FALSE, 0);

    ok_hbox = gtk_hbutton_box_new();
    gtk_button_box_set_layout(GTK_BUTTON_BOX(ok_hbox), GTK_BUTTONBOX_SPREAD);
    gtk_button_box_set_spacing(GTK_BUTTON_BOX(ok_hbox), 5);
    gtk_box_pack_start(GTK_BOX(all_vbox), ok_hbox, FALSE, FALSE, 0);
    ok_but = gtk_button_new_with_label("OK");
    gtk_signal_connect_object(GTK_OBJECT(ok_but), "clicked",
                              gtk_widget_destroy, GTK_OBJECT(window));
    GTK_WIDGET_SET_FLAGS(ok_but, GTK_CAN_DEFAULT);
    gtk_box_pack_start(GTK_BOX(ok_hbox), ok_but, TRUE, TRUE, 0);
    gtk_widget_grab_default(ok_but);

    gtk_widget_show_all(window);
  }

  title = g_strdup_printf("File Info - %s", g_basename(filename));
  gtk_window_set_title(GTK_WINDOW(window), title);
  g_free(title);

  gtk_entry_set_text(GTK_ENTRY(filename_entry), filename);

  if (m4a_info) {
    moov = m4a_info->moov;
  } else {
    FILE *fh;

    if ((fh = fopen(filename, "rb")) == NULL) return;
    if ((moov = g_malloc0(sizeof(moov_t))) == NULL) {
      fclose(fh);
      return;
    }
    init_moov(moov);
    if (!get_moov(fh, moov)) {
      clear_moov(moov);
      g_free(moov);
      fclose(fh);
      return;
    }
    fclose(fh);
  }

  if (moov->udta.title) {
    title = g_strdup(moov->udta.title);
  } else {
    title = g_strdup(g_basename(filename));
    if ((tmp = strrchr(title, '.')) != NULL) *tmp = '\0';
  }
  gtk_label_set_text(GTK_LABEL(title_entry), title);
  g_free(title);

  if (moov->udta.album) {
    title = g_strdup(moov->udta.album);
    gtk_label_set_text(GTK_LABEL(album_entry), title);
    g_free(title);
    gtk_widget_show(GTK_WIDGET(album_label));
    gtk_widget_show(GTK_WIDGET(album_entry));
  } else {
    gtk_label_set_text(GTK_LABEL(album_entry), "");
    gtk_widget_hide(GTK_WIDGET(album_label));
    gtk_widget_hide(GTK_WIDGET(album_entry));
  }

  if (moov->udta.artist) {
    title = g_strdup(moov->udta.artist);
    gtk_label_set_text(GTK_LABEL(artist_entry), title);
    g_free(title);
    gtk_widget_show(GTK_WIDGET(artist_label));
    gtk_widget_show(GTK_WIDGET(artist_entry));
  } else {
    gtk_label_set_text(GTK_LABEL(artist_entry), "");
    gtk_widget_hide(GTK_WIDGET(artist_label));
    gtk_widget_hide(GTK_WIDGET(artist_entry));
  }

  if (moov->udta.composer) {
    title = g_strdup(moov->udta.composer);
    gtk_label_set_text(GTK_LABEL(composer_entry), title);
    g_free(title);
    gtk_widget_show(GTK_WIDGET(composer_label));
    gtk_widget_show(GTK_WIDGET(composer_entry));
  } else {
    gtk_label_set_text(GTK_LABEL(composer_entry), "");
    gtk_widget_hide(GTK_WIDGET(composer_label));
    gtk_widget_hide(GTK_WIDGET(composer_entry));
  }

  if (moov->udta.tracknum) {
    title = g_strdup_printf("Track %d", moov->udta.tracknum);
    if (moov->udta.tracks) {
      char *tk_tmp;

      tmp = g_strdup_printf(" of %d", moov->udta.tracks);
      tk_tmp = g_strconcat(title, tmp, NULL);
      g_free(title);
      g_free(tmp);
      title = tk_tmp;
    }
    gtk_label_set_text(GTK_LABEL(track_info), title);
    gtk_widget_show(GTK_WIDGET(track_info));
    g_free(title);
  } else if (moov->udta.tracks) {
    title = g_strdup_printf("%d tracks", moov->udta.tracks);
    gtk_label_set_text(GTK_LABEL(track_info), title);
    gtk_widget_show(GTK_WIDGET(track_info));
    g_free(title);
  } else {
    gtk_label_set_text(GTK_LABEL(track_info), "");
    gtk_widget_hide(GTK_WIDGET(track_info));
  }

  if (moov->udta.date) {
    title = g_strdup(moov->udta.date);
    gtk_label_set_text(GTK_LABEL(year_entry), title);
    g_free(title);
    gtk_widget_show(GTK_WIDGET(year_label));
    gtk_widget_show(GTK_WIDGET(year_entry));
  } else {
    gtk_label_set_text(GTK_LABEL(year_entry), "");
    gtk_widget_hide(GTK_WIDGET(year_label));
    gtk_widget_hide(GTK_WIDGET(year_entry));
  }

  if (moov->udta.discnum) {
    title = g_strdup_printf("Disc %d", moov->udta.discnum);
    if (moov->udta.discs) {
      char *dk_tmp;

      tmp = g_strdup_printf(" of %d", moov->udta.discs);
      dk_tmp = g_strconcat(title, tmp, NULL);
      g_free(title);
      g_free(tmp);
      title = dk_tmp;
    }
    gtk_label_set_text(GTK_LABEL(disc_info), title);
    gtk_widget_show(GTK_WIDGET(disc_info));
    g_free(title);
  } else if (moov->udta.discs) {
    title = g_strdup_printf("%d discs", moov->udta.discs);
    gtk_label_set_text(GTK_LABEL(disc_info), title);
    gtk_widget_show(GTK_WIDGET(disc_info));
    g_free(title);
  } else {
    gtk_label_set_text(GTK_LABEL(disc_info), "");
    gtk_widget_hide(GTK_WIDGET(disc_info));
  }

  if (moov->udta.is_comp) {
    gtk_widget_show(GTK_WIDGET(is_comp_entry));
  } else {
    gtk_widget_hide(GTK_WIDGET(is_comp_entry));
  }

  if (moov->udta.genre) {
    title = g_strdup(moov->udta.genre);
    gtk_label_set_text(GTK_LABEL(genre_entry), title);
    g_free(title);
    gtk_widget_show(GTK_WIDGET(genre_label));
    gtk_widget_show(GTK_WIDGET(genre_entry));
  } else {
    gtk_label_set_text(GTK_LABEL(genre_entry), "");
    gtk_widget_hide(GTK_WIDGET(genre_label));
    gtk_widget_hide(GTK_WIDGET(genre_entry));
  }

  gtk_text_set_point(GTK_TEXT(comment_entry), 0);
  gtk_text_forward_delete(GTK_TEXT(comment_entry),
                          gtk_text_get_length(GTK_TEXT(comment_entry)));
  if (moov->udta.comment) {
    title = g_strdup(moov->udta.comment);
    gtk_text_set_point(GTK_TEXT(comment_entry), 0);
    gtk_text_insert(GTK_TEXT(comment_entry), NULL, NULL, NULL,
                    title, -1);
    gtk_widget_show(GTK_WIDGET(comment_label));
    gtk_widget_show(GTK_WIDGET(comment_entry));
  } else {
    gtk_widget_hide(GTK_WIDGET(comment_label));
    gtk_widget_hide(GTK_WIDGET(comment_entry));
  }

  if (m4a_info) {
    title = g_strdup_printf("Bitrate: %d kb/s", m4a_info->bitrate / 1000);
    gtk_label_set_text(GTK_LABEL(file_bitrate), title);
    g_free(title);

    title = g_strdup_printf("Samplerate: %d Hz", m4a_info->samplerate);
    gtk_label_set_text(GTK_LABEL(file_samplerate), title);
    g_free(title);

    title = g_strdup_printf("%s", ((m4a_info->channels - 1) ?
                                   "Stereo" : "Mono"));
    gtk_label_set_text(GTK_LABEL(file_channels), title);
    g_free(title);

    title = g_strdup_printf("%d samples", m4a_info->samples);
    gtk_label_set_text(GTK_LABEL(file_samples), title);
    g_free(title);

    gtk_widget_show(GTK_WIDGET(file_frame));
  } else {
    gtk_label_set_text(GTK_LABEL(file_bitrate), "");
    gtk_label_set_text(GTK_LABEL(file_samplerate), "");
    gtk_label_set_text(GTK_LABEL(file_channels), "");
    gtk_label_set_text(GTK_LABEL(file_samples), "");

    gtk_widget_hide(GTK_WIDGET(file_frame));
  }
}
