/*
 * Slarti's libm4a.so Input plugin for xmms
 * Chris Pinard <slarti@gweep.net>, 20 Feb 2004
 * Based heavily on mp4Plugin and xmms' vorbis plugin
 * Also pulling from the quicktime4linux library, which looks
 * a whole less sucky than the C++ shit of mpeg4ip's mp4v2 lib.
 * This improves on mp4Plugin in some critical ways, like having actual
 * support for m4a's id3-like tags, being able to actually show the
 * sample rate, and not chopping the last four seconds off of the end of
 * playing a file. Of course, I don't actually know anything about audio
 * programming, and I should be sleeping and looking for a job while I
 * work on this instead, but what the hell, we'll see what we can come
 * up with.
 */

#include <src/libm4a.h>

InputPlugin m4a_ip =
{
  NULL, /* Handle. Provided by xmms */
  NULL, /* Filename. Provided by xmms */
  NULL, /* Description for prefbox. Filled in in get_iplugin_info */
  m4a_init, /* Init function to call on plugin load */
  m4a_about, /* Show the about box */
  NULL, /* configure -- Configuration */
  is_m4a_file, /* Can this plugin handle this file? */
  NULL, /* scan_dir -- No clue. xmms src points to Input/cdaudio */
  play_m4a, /* Play file. */
  stop_m4a, /* Stop playing */
  pause_m4a, /* Pause/unpause playing */
  seek_m4a, /* Seek to a given time */
  NULL, /* set_eq -- Set the equalizer. Probably not gonna do this */
  m4a_get_time, /* Get the time, return output plugin's output time */
  NULL, /* get_volume -- Let output plugin handle volume */
  NULL, /* set_volume -- See get_volume */
  0, /* cleanup -- Called when xmms exits. Unsure about this one */
  NULL, /* get_vis_type -- Obsolete. Don't use */
  NULL, /* add_vis_pcm -- Send data to visualization plugins */
  NULL, /* set_info -- Fill in player window info. Provided by xmms */
  NULL, /* set_info_text -- Show some text in title box. Provided by xmms */
  get_m4a_info, /* Grab title string */
  m4a_info_box, /* Bring up info window for a given filename */
  NULL /* output -- Handle to output plugin. Provided by xmms */
};

player_t *m4a_info = NULL;
static pthread_t *decode_thread;

InputPlugin *get_iplugin_info(void)
{
  m4a_ip.description=g_strdup("Slarti's mp4/aac/m4a Plugin");
  return &m4a_ip;
}

static void m4a_init(void)
{
#ifdef DEBUG
  g_print("m4a_init called.\n");
#endif
  /* Create decode thread structure, set to 0s */
  if ((decode_thread = g_malloc0(sizeof(pthread_t))) == NULL) {
    g_print("Unable to allocate memory for decode thread!\n");
  }
  return; /* Dunno what I'm going to feel I need to do here yet.
             Possibly set up some config options stuff, if I ever
             do any actual config options stuff. */
}

static void m4a_about(void)
{
  static GtkWidget *aboutbox;

#ifdef DEBUG
  g_print("m4a_about called\n");
#endif
  if (aboutbox != NULL)
    return;

  aboutbox = xmms_show_message("About Slarti's m4a plugin",
                               "Based on xmms' vorbis plugin\n"
                               "and mp4Plugin. Uses libfaad.\n"
                               "Chris Pinard <slarti@gweep.net>",
                               "Ok", FALSE, NULL, NULL);
  gtk_signal_connect(GTK_OBJECT(aboutbox), "destroy",
                     GTK_SIGNAL_FUNC(gtk_widget_destroyed), &aboutbox);
}

static int is_m4a_file(char *filename)
{
#ifdef DEBUG
  g_print("is_m4a_file called. Filename: %s\n", filename);
#endif
  return is_mp4(filename);
}

static void *decode_mp4(void *arg)
{
  char *filename = arg;
  int tk;
  faacDecHandle m4aDec;
  faacDecFrameInfo frameInfo;
  unsigned char *aacBuffer = 0;
  unsigned char *decBuffer = 0;
  guint sample = 0;
  guint samplesize = 0;
  esds_t *esds;
  guint song_length;
  char *song_title;

#ifdef DEBUG
  g_print("decode_mp4 called. Playing file %s.\n", filename);
#endif
  /* Open the file. */
  if ((m4a_info->fh = fopen(filename, "rb")) == 0) {
    g_print("Unable to open file %s.\n", filename);
    fclose(m4a_info->fh);
    m4a_info->playing = FALSE;
    m4a_info->eof = TRUE;
    pthread_exit(NULL);
  }
#ifdef DEBUG
  g_print("decode_mp4: File opened.\n");
#endif
  /* Get the moov atom which has the metadata */
  if (!get_moov(m4a_info->fh, m4a_info->moov)) {
    g_print("Unable to find moov atom in file %s.\n", filename);
    fclose(m4a_info->fh);
    m4a_info->playing = FALSE;
    m4a_info->eof = TRUE;
    pthread_exit(NULL);
  }
#ifdef DEBUG
  g_print("decode_mp4: moov atom acquired.\n");
#endif
  /* Get the track to play from the moov */
  if ((tk = get_audio_track(m4a_info->moov)) == -1) {
    g_print("No audio tracks to play!\n");
    fclose(m4a_info->fh);
    m4a_info->playing = FALSE;
    m4a_info->eof = TRUE;
    pthread_exit(NULL);
  } else {
#ifdef DEBUG
    g_print("decode_mp4: Got audio track. (%d)\n", tk);
#endif
    m4a_info->trak = m4a_info->moov->trak[tk];
  }
  /* Get the ESDS, which will have info on the decoder config and bitrate */
  esds = &(m4a_info->trak->mdia.minf.stbl.stsd.t[0].esds);
  if ((!esds->got_esds) && (!esds->decConfig)) {
    g_print("No ESDS for decoder config!\n");
    fclose(m4a_info->fh);
    m4a_info->playing = FALSE;
    m4a_info->eof = TRUE;
    pthread_exit(NULL);
  }
  m4a_info->bitrate = esds->avg_bitrate;
#ifdef DEBUG
  g_print("decode_mp4: Got ESDS. Average bitrate %d.\n", m4a_info->bitrate);
#endif
  /* Open the AAC decoder */
  m4aDec = faacDecOpen();
#ifdef DEBUG
  g_print("decode_mp4: AAC decoder opened.\n");
#endif
  /* And initialize it */
  if (faacDecInit2(m4aDec, esds->decConfig, esds->dConfLen,
                   &(m4a_info->samplerate), &(m4a_info->channels)) < 0) {
    g_print("Unable to configure decoder!\n");
    faacDecClose(m4aDec);
    fclose(m4a_info->fh);
    m4a_info->playing = FALSE;
    m4a_info->eof = TRUE;
    pthread_exit(NULL);
  }
  if (!m4a_info->channels) {
    g_print("No channels! Not allowed!\n");
    faacDecClose(m4aDec);
    fclose(m4a_info->fh);
    m4a_info->playing = FALSE;
    m4a_info->eof = TRUE;
    pthread_exit(NULL);
  }
#ifdef DEBUG
  g_print("decode_mp4: AAC decoder configured. Samplerate %d, %d channels.\n",
          m4a_info->samplerate, m4a_info->channels);
#endif

  /* Okay. The decoder's all set. Now get the number of samples, and the
     song's title and duration for handing to set_info */
  m4a_info->samples = m4a_info->trak->mdia.minf.stbl.stsz.entries;
#ifdef DEBUG
  g_print("decode_mp4: %d samples.\n", m4a_info->samples);
#endif
  song_length = get_song_time(m4a_info->moov);
#ifdef DEBUG
  g_print("decode_mp4: Song is %d milliseconds long.\n", song_length);
#endif
  song_title = get_song_title(m4a_info->moov, filename);
#ifdef DEBUG
  g_print("decode_mp4: Song title %s\n", song_title);
#endif
  m4a_ip.set_info(song_title, song_length, m4a_info->bitrate,
                  m4a_info->samplerate, m4a_info->channels);
#ifdef DEBUG
  g_print("decode_mp4: Info set.\n");
#endif
  m4a_ip.output->open_audio(FMT_S16_NE, m4a_info->samplerate,
                            m4a_info->channels);
#ifdef DEBUG
  g_print("decode_mp4: Audio opened.\n");
#endif

  /* Last thing, honest. Gotta seek to the location of the first sample. */
#ifdef DEBUG
  g_print("decode_mp4: Seeking to sample 0.\n");
#endif
  seek_to_sample(m4a_info->fh, m4a_info->trak, 0);

  /* And now, we are ready to play. */
#ifdef DEBUG
  g_print("decode_mp4: Entering play loop.\n");
#endif
  while (m4a_info->playing) {
#ifdef DEBUG
    g_print("decode_mp4: Loop start.\n");
#endif
    /* Are we being asked to seek to a new point in the song? */
    if (m4a_info->time_index != -1) {
#ifdef DEBUG
      g_print("decode_mp4: Seeking to time %d. (Song time index %d)\n",
              m4a_info->time_index,
              (m4a_info->time_index * m4a_info->samplerate));
#endif
      sample = time_to_sample(m4a_info->time_index * m4a_info->samplerate,
                              m4a_info->trak);
#ifdef DEBUG
      g_print("decode_mp4: Seeking sample %d.\n", sample);
#endif
      seek_to_sample(m4a_info->fh, m4a_info->trak, sample);
      m4a_ip.output->flush(m4a_info->time_index * 1000);
      m4a_info->time_index = -1;
    }

    /* Get a sample, if we still have samples to get. */
    if (!m4a_info->eof) {
      samplesize = m4a_info->trak->mdia.minf.stbl.stsz.t[sample].size;
#ifdef DEBUG
      g_print("decode_mp4: Playing sample %d, size %d.\n",
              sample, samplesize);
#endif

      if ((aacBuffer = g_malloc0(samplesize)) == NULL) {
        g_print("Unable to allocate space for sample %u\n", sample);
        m4a_ip.output->buffer_free();
        m4a_ip.output->close_audio();
        faacDecClose(m4aDec);
        fclose(m4a_info->fh);
        m4a_info->playing = FALSE;
        m4a_info->eof = TRUE;
        pthread_exit(NULL);
      }

      if ((fread(aacBuffer, 1, samplesize, m4a_info->fh)) < samplesize) {
        g_print("Unable to read sample %u\n", sample);
        m4a_ip.output->buffer_free();
        m4a_ip.output->close_audio();
        faacDecClose(m4aDec);
        fclose(m4a_info->fh);
        m4a_info->playing = FALSE;
        m4a_info->eof = TRUE;
        pthread_exit(NULL);
      }

      decBuffer = faacDecDecode(m4aDec, &frameInfo, aacBuffer, samplesize);
      if (frameInfo.error > 0) {
        g_print("Decoder error: %s\n", faacDecGetErrorMessage(frameInfo.error));
        m4a_ip.output->buffer_free();
        m4a_ip.output->close_audio();
        faacDecClose(m4aDec);
        fclose(m4a_info->fh);
        m4a_info->playing = FALSE;
        m4a_info->eof = TRUE;
        pthread_exit(NULL);
      }

      if (aacBuffer) {
        g_free(aacBuffer);
        aacBuffer = NULL;
        samplesize = 0;
      }
      sample++;
      if (sample >= m4a_info->samples) {
        m4a_info->eof = TRUE;
      }

      m4a_ip.add_vis_pcm(m4a_ip.output->written_time(), FMT_S16_NE,
                         m4a_info->channels, frameInfo.samples << 1,
                         decBuffer);
      while ((m4a_ip.output->buffer_free() < (frameInfo.samples << 1)) &&
             (m4a_info->playing) && (m4a_info->time_index == -1)) {
        xmms_usleep(10000);
      }
      m4a_ip.output->write_audio(decBuffer, frameInfo.samples << 1);
    } else {
      /* All the samples are got. Give it a rest before cleanup. */
      xmms_usleep(10000);
    }
  }

  /* Done playing. Clean stuff up. */
  g_free(song_title);
  fclose(m4a_info->fh);
  m4a_ip.output->close_audio();
  faacDecClose(m4aDec);
  m4a_info->playing = FALSE;
  g_free(filename);
  pthread_exit(NULL);
}

static void play_m4a(char *filename)
{
#ifdef DEBUG
  g_print("play_m4a called. Playing %s.\n", filename);
#endif
  if ((m4a_info = g_malloc0(sizeof(player_t))) == NULL) {
#ifdef DEBUG
    g_print("play_m4a: Uh, crap! Failed to malloc m4a_info!\n");
#endif
  }
  if ((m4a_info->moov = g_malloc0(sizeof(moov_t))) == NULL) {
    g_print("play_m4a: Crap! Failed to malloc moov atom!\n");
  }
  init_moov(m4a_info->moov);
  m4a_info->playing = TRUE;
  m4a_info->time_index = -1;
  m4a_info->eof = FALSE;
#ifdef DEBUG
  g_print("play_m4a: Initial config complete.\n");
#endif

  pthread_create(decode_thread, NULL, decode_mp4, g_strdup(filename));
#ifdef DEBUG
  g_print("play_m4a: Thread created.\n");
#endif
}

static void stop_m4a(void)
{
#ifdef DEBUG
  g_print("stop_m4a called.\n");
#endif
  if (m4a_info && m4a_info->playing) {
    m4a_info->playing = FALSE;
    pthread_join(*decode_thread, NULL);
    clear_moov(m4a_info->moov);
    g_free(m4a_info->moov);
    g_free(m4a_info);
    m4a_info = NULL;
#ifdef DEBUG
    g_print("stop_m4a: Stopping complete.\n");
#endif
  }
}

static void pause_m4a(short flag)
{
#ifdef DEBUG
  g_print("pause_m4a called. Setting pausing to %d.\n", flag);
#endif
  m4a_ip.output->pause(flag);
}

static void seek_m4a(int time)
{
#ifdef DEBUG
  g_print("seek_m4a called. Seeking to %d.\n", time);
#endif
  m4a_info->time_index = time;
  while ((m4a_info->playing) && (m4a_info->time_index != -1)) {
#ifdef DEBUG
    g_print("seek_m4a: waiting...\n");
#endif
    xmms_usleep(10000);
  }
}

static int m4a_get_time(void)
{
#ifdef DEBUG
  g_print("m4a_get_time called.\n");
#endif
  if (! m4a_info) {
#ifdef DEBUG
    g_print("m4a_get_time: No m4a_info.\n");
#endif
    return -1;
  }
  if (!m4a_info->playing) {
#ifdef DEBUG
    g_print("m4a_get_time: Not playing.\n");
#endif
    return -1;
  }
  if (m4a_info->eof) {
#ifdef DEBUG
    g_print("m4a_get_time: end of file\n");
#endif
    if (!(m4a_ip.output->buffer_playing())) {
#ifdef DEBUG
      g_print("m4a_get_time: And no buffer playing.\n");
#endif
      return -1;
    }
  }
#ifdef DEBUG
  g_print("m4a_get_time: Sending time info to xmms.\n");
#endif
  return m4a_ip.output->output_time();
}

static void get_m4a_info(char *filename, char **song_title, int *song_len)
{
  FILE *fh;
  moov_t moov;

#ifdef DEBUG
  g_print("get_m4a_info called\n");
#endif
  /* Set defaults in case there's no info for it here. */
  (*song_len) = -1;
  (*song_title) = NULL;

  if ((fh = fopen(filename, "rb")) != NULL) {
#ifdef DEBUG
    g_print("get_m4a_info: File opened\n");
#endif
    /* Initialize moov atom */
    init_moov(&moov);
    if (get_moov(fh, &moov)) {
#ifdef DEBUG
      g_print("get_m4a_info: Got moov atom\n");
#endif
      /* Get filename and length info */
      (*song_len) = get_song_time(&moov);
#ifdef DEBUG
      g_print("get_m4a_info: Song length %d\n", (*song_len));
#endif

      (*song_title) = get_song_title(&moov, filename);
    }
    fclose(fh);
  }
}
