#include <src/libm4a.h>

guint get_song_time(moov_t *moov)
{
  mdhd_t *mdhd;
  int i;

#ifdef DEBUG
  g_print("get_song_time called\n");
#endif
  /* Find the first audio track */
  if ((i = get_audio_track(moov)) == -1) { /* No audio tracks? */
    return -1; /* Punt. */
  }
  mdhd = &(moov->trak[i]->mdia.mdhd);
#ifdef DEBUG
  g_print("Duration %u, Timescale %u\n", mdhd->duration, mdhd->timescale);
#endif
  return ((mdhd->duration / mdhd->timescale) * 1000);
}

gchar *get_ext(const char *filename)
{
  gchar *ext = strrchr(filename, '.');
  if (ext != NULL) {
    ext++;
  }
  return ext;
}

static char *getstr(char *str)
{
  if ((str) && (strlen(str) > 0)) {
    return str;
  } else {
    return NULL;
  }
}

gchar *get_song_title(moov_t *moov, char *filename)
{
  gchar *ret = NULL, *path, *temp;
  TitleInput *input;
  GDate *song_date;

  XMMS_NEW_TITLEINPUT(input);

#ifdef DEBUG
  g_print("get_song_title called\n");
#endif
  /* Fill in filename-based parameters */
  path = g_strdup(filename);
  temp = strrchr(path, '/');
  if (temp) {
    *temp = '\0'; /* Chop filename off of path */
  }
  input->file_name = g_basename(filename);
  input->file_path = g_strdup_printf("%s/", path);
  input->file_ext = get_ext(filename);
#ifdef DEBUG
  g_print("get_song_title: File path info entered.\n");
#endif

  /* Now fill in information from udta atom */
  input->performer = getstr(moov->udta.artist);
  input->album_name = getstr(moov->udta.album);
  input->track_name = getstr(moov->udta.title);
  input->track_number = moov->udta.tracknum;
#ifdef DEBUG
  g_print("get_song_title: Initial udta fields entered.\n");
#endif
  /* Do something about the year/date fields */
  if (getstr(moov->udta.date)) {
    song_date = g_date_new();
#ifdef DEBUG
    g_print("get_song_title: GDate allocated\n");
#endif
    g_date_set_parse(song_date, moov->udta.date);
#ifdef DEBUG
    g_print("get_song_title: Date parsed\n");
#endif
    if (g_date_valid(song_date)) {
#ifdef DEBUG
      g_print("get_song_title: Date valid\n");
#endif
      input->year = g_date_year(song_date);
    }
#ifdef DEBUG
    g_print("get_song_title: Year added: %d\n", input->year);
#endif
    input->date = getstr(moov->udta.date);
#ifdef DEBUG
    g_print("get_song_title: Year/Date added.\n");
#endif
  }
  input->genre = getstr(moov->udta.genre);
  input->comment = getstr(moov->udta.comment);
#ifdef DEBUG
  g_print("get_song_title: Final fields entered.\n");
#endif

  /* Input fields are all full. Try to format a track name */
  ret = xmms_get_titlestring(xmms_get_gentitle_format(), input);
  g_free(input);
  g_free(path);
#ifdef DEBUG
  g_print("get_song_title: Track name formatted.\n");
#endif

  if (!ret) { /* Title formatting failed for some reason. */
#ifdef DEBUG
    g_print("get_song_title: Had to go to filename-based naming.\n");
#endif
    /* Format according to filename. */
    ret = g_strdup(g_basename(filename)); /* Get filename w/out path */
    if (get_ext(ret) != NULL) {
      *(get_ext(ret) - 1) = '\0'; /* Chop off extension */
    }
  }

#ifdef DEBUG
  g_print("get_song_title complete: '%s'\n", ret);
#endif
  return ret; /* Hand over completed title */
}
